import argparse
import ast
import json
import os
import re
import tqdm

from .simpledicomanonymizer import *

def anonymize(inputPath, outputPath, anonymizationActions):
    # Get input arguments
    InputFolder = ''
    OutputFolder = ''

    if os.path.isdir(inputPath):
        InputFolder = inputPath

    if os.path.isdir(outputPath):
        OutputFolder = outputPath
        if InputFolder == '':
            outputPath = OutputFolder + os.path.basename(inputPath)

    if InputFolder != '' and OutputFolder == '':
        print('Error, please set an output folder path with an input folder path')
        sys.exit()

    # Generate list of input file if a folder has been set
    inputFilesList = []
    outputFilesList = []
    if InputFolder == '':
        inputFilesList.append(inputPath)
        outputFilesList.append(outputPath)
    else:
        files = os.listdir(InputFolder)
        for fileName in files:
            inputFilesList.append(InputFolder + '/' + fileName)
            outputFilesList.append(OutputFolder + '/' + fileName)

    progressBar = tqdm.tqdm(total=len(inputFilesList))
    for cpt in range(len(inputFilesList)):
        anonymizeDICOMFile(inputFilesList[cpt], outputFilesList[cpt], anonymizationActions)
        progressBar.update(1)

    progressBar.close()


def generateActionsDictionary(mapActionTag, definedActionMap = {}):
    generatedMap = {}
    cpt = 0
    for tag in mapActionTag:
        test = [tag]
        action = mapActionTag[tag]

        # Define the associated function to the tag
        if callable(action):
            actionFunction = action
        else:
            actionFunction = definedActionMap[action] if action in definedActionMap else eval(action)

        # Generate the map
        if cpt == 0:
            generatedMap = generateActions(test, actionFunction)
        else:
            generatedMap.update(generateActions(test, actionFunction))
        cpt += 1

    return generatedMap


def main(definedActionMap = {}):
    parser = argparse.ArgumentParser(add_help=True)
    parser.add_argument('input', help='Path to the input dicom file or input directory which contains dicom files')
    parser.add_argument('output', help='Path to the output dicom file or output directory which will contains dicom files')
    parser.add_argument('-t', action='append', nargs='*', help='tags action : Defines a new action to apply on the tags list')
    parser.add_argument('--dictionary', action='store', help='File which contains a dictionary that can be added to the original one')
    args = parser.parse_args()

    InputPath = args.input
    OutputPath = args.output

    # Create a new actions' dictionary from parameters
    newAnonymizationActions = {}
    cpt = 0
    if args.t:
        numberOfNewTagsActions = len(args.t)
        if numberOfNewTagsActions > 0:
            for i in range(numberOfNewTagsActions):
                actionName = args.t[i].pop()
                if len(args.t[i]) == 0:
                    continue
                tagsList = []
                for tag in args.t[i]:
                    tagsList.append(ast.literal_eval(tag))

                if cpt == 0:
                    newAnonymizationActions = generateActions(tagsList, eval(actionName))
                else:
                    newAnonymizationActions.update(generateActions(tagsList, eval(actionName)))
                cpt += 1

    # Read an existing dictionary
    if args.dictionary:
        with open(args.dictionary) as json_file:
            data = json.load(json_file)
            for k, v in data.items():
                l = [ast.literal_eval(k)]
                actionFunction = definedActionMap[v] if v in definedActionMap else eval(v)
                if cpt == 0:
                    newAnonymizationActions = generateActions(l, actionFunction)
                else:
                    newAnonymizationActions.update(generateActions(l, actionFunction))
                cpt += 1

    # Launch the anonymization
    anonymize(InputPath, OutputPath, newAnonymizationActions)



# Tags anonymized in DICOM standard
# Documentation for groups meaning can be found in default associated actions.
# http://dicom.nema.org/dicom/2013/output/chtml/part15/chapter_E.html#table_E.1-1

# Replaced tags
D_TAGS = [
    (0x0070, 0x0001), # Graphic Annotation Sequence
    (0x0040, 0x1101), # Person Identification Code Sequence
    (0x0040, 0xA123), # Person Name
    (0x0040, 0xA075), # Verifying Observer Name
    (0x0040, 0xA073), # Verifying Observer Sequence
]

# Replaced with empty values (0, '', ...)
Z_TAGS = [
    (0x0008, 0x0050), # Accession Number
    (0x0070, 0x0084), # Content Creator's Name
    (0x0040, 0x2017), # Filler Order Number / Imaging Service Request
    (0x0010, 0x0020), # Patient ID
    (0x0010, 0x0030), # Patient's Birth Date
    (0x0010, 0x0010), # Patient's Name
    (0x0010, 0x0040), # Patient's Sex
    (0x0040, 0x2016), # Placer Order Number / Imaging Service Request
    (0x0008, 0x0090), # Referring Physician's Name
    (0x0008, 0x0020), # Study Date
    (0x0020, 0x0010), # Study ID
    (0x0008, 0x0030), # Study Time
    (0x0040, 0xA088), # Verifying Observer Identification Code Sequence
]

# Deleted tags
X_TAGS = [
    (0x0018, 0x4000), # Acquisition Comments
    (0x0040, 0x0555), # Acquisition Context Sequence
    (0x0018, 0x9424), # Acquisition Protocol Description
    (0x0040, 0x4035), # Actual Human Performers Sequence
    (0x0010, 0x21B0), # Additional Patient's History
    (0x0038, 0x0010), # Admission ID
    (0x0038, 0x0020), # Admitting Date
    (0x0008, 0x1084), # Admitting Diagnoses Code Sequence
    (0x0008, 0x1080), # Admitting Diagnoses Description
    (0x0038, 0x0021), # Admitting Time
    (0x0000, 0x1000), # Affected SOP Instance UID
    (0x0010, 0x2110), # Allergies
    (0x4000, 0x0010), # Arbitrary
    (0x0040, 0xA078), # Author Observer Sequence
    (0x0010, 0x1081), # Branch of Service
    (0x0018, 0x1007), # Cassette ID
    (0x0040, 0x0280), # Comments on the Performed Procedure Step
    (0x0040, 0x3001), # Confidentiality Constraint on Patient Data Description
    (0x0070, 0x0086), # Content Creator's Identification Code Sequence
    (0x0040, 0xA730), # Content Sequence
    (0x0018, 0xA003), # Contribution Description
    (0x0010, 0x2150), # Country of Residence
    (0x0038, 0x0300), # Current Patient Location
    (0x5000, 0x0000, 0xFF00, 0x0000), # Curve Data
    (0x0008, 0x0025), # Curve Date
    (0x0008, 0x0035), # Curve Time
    (0x0040, 0xA07C), # Custodial Organization Sequence
    (0xFFFC, 0xFFFC), # Data Set Trailing Padding
    (0x0008, 0x2111), # Derivation Description
    (0x0400, 0x0100), # Digital Signature UID
    (0xFFFA, 0xFFFA), # Digital Signatures Sequence
    (0x0038, 0x0040), # Discharge Diagnosis Description
    (0x4008, 0x011A), # Distribution Address
    (0x4008, 0x0119), # Distribution Name
    (0x0010, 0x2160), # Ethnic Group
    (0x0020, 0x9158), # Frame Comments
    (0x0018, 0x1008), # Gantry ID
    (0x0018, 0x1005), # Generator ID
    (0x0040, 0x4037), # Human Performers Name
    (0x0040, 0x4036), # Human Performers Organization
    (0x0088, 0x0200), # Icon Image Sequence(see Note 12)
    (0x0008, 0x4000), # Identifying Comments
    (0x0020, 0x4000), # Image Comments
    (0x0028, 0x4000), # Image Presentation Comments
    (0x0040, 0x2400), # Imaging Service Request Comments
    (0x4008, 0x0300), # Impressions
    (0x0008, 0x0081), # Institution Address
    (0x0008, 0x1040), # Institutional Department Name
    (0x0010, 0x1050), # Insurance Plan Identification
    (0x0040, 0x1011), # Intended Recipients of Results Identification Sequence
    (0x4008, 0x0111), # Interpretation Approver Sequence
    (0x4008, 0x010C), # Interpretation Author
    (0x4008, 0x0115), # Interpretation Diagnosis Description
    (0x4008, 0x0202), # Interpretation ID Issuer
    (0x4008, 0x0102), # Interpretation Recorder
    (0x4008, 0x010B), # Interpretation Text
    (0x4008, 0x010A), # Interpretation Transcriber
    (0x0038, 0x0011), # Issuer of Admission ID
    (0x0010, 0x0021), # Issuer of Patient ID
    (0x0038, 0x0061), # Issuer of Service Episode ID
    (0x0010, 0x21D0), # Last Menstrual Date
    (0x0400, 0x0404), # MAC
    (0x0010, 0x2000), # Medical Alerts
    (0x0010, 0x1090), # Medical Record Locator
    (0x0010, 0x1080), # Military Rank
    (0x0400, 0x0550), # Modified Attributes Sequence
    (0x0020, 0x3406), # Modified Image Description
    (0x0020, 0x3401), # Modifying Device ID
    (0x0020, 0x3404), # Modifying Device Manufacturer
    (0x0008, 0x1060), # Name of Physician(s) Reading Study
    (0x0040, 0x1010), # Names of Intended Recipient of Results
    (0x0010, 0x2180), # Occupation
    (0x0400, 0x0561), # Original Attributes Sequence
    (0x0040, 0x2010), # Order Callback Phone Number
    (0x0040, 0x2008), # Order Entered By
    (0x0040, 0x2009), # Order Enterer Location
    (0x0010, 0x1000), # Other Patient IDs
    (0x0010, 0x1002), # Other Patient IDs Sequence
    (0x0010, 0x1001), # Other Patient Names
    (0x6000, 0x4000, 0xFF00, 0xFFFF), # Overlay Comments
    (0x6000, 0x3000, 0xFF00, 0xFFFF), # Overlay Data
    (0x0008, 0x0024), # Overlay Date
    (0x0008, 0x0034), # Overlay Time
    (0x0040, 0xA07A), # Participant Sequence
    (0x0010, 0x1040), # Patient Address
    (0x0010, 0x4000), # Patient Comments
    (0x0038, 0x0500), # Patient State
    (0x0040, 0x1004), # Patient Transport Arrangements
    (0x0010, 0x1010), # Patient's Age
    (0x0010, 0x1005), # Patient's Birth Name
    (0x0010, 0x0032), # Patient's Birth Time
    (0x0038, 0x0400), # Patient's Institution Residence
    (0x0010, 0x0050), # Patient's Insurance Plan Code Sequence
    (0x0010, 0x1060), # Patient's Mother's Birth Name
    (0x0010, 0x0101), # Patient's Primary Language Code Sequence
    (0x0010, 0x0102), # Patient's Primary Language Modifier Code Sequence
    (0x0010, 0x21F0), # Patient's Religious Preference
    (0x0010, 0x1020), # Patient's Size
    (0x0010, 0x2154), # Patient's Telephone Numbers
    (0x0010, 0x1030), # Patient's Weight
    (0x0040, 0x0243), # Performed Location
    (0x0040, 0x0254), # Performed Procedure Step Description
    (0x0040, 0x0250), # Performed Procedure Step End Date
    (0x0040, 0x0251), # Performed Procedure Step End Time
    (0x0040, 0x0253), # Performed Procedure Step ID
    (0x0040, 0x0244), # Performed Procedure Step Start Date
    (0x0040, 0x0245), # Performed Procedure Step Start Time
    (0x0040, 0x0241), # Performed Station AE Title
    (0x0040, 0x4030), # Performed Station Geographic Location Code Sequence
    (0x0040, 0x0242), # Performed Station Name
    (0x0040, 0x4028), # Performed Station Name Code Sequence
    (0x0008, 0x1052), # Performing Physician Identification Sequence
    (0x0008, 0x1050), # Performing Physicians' Name
    (0x0040, 0x1102), # Person Address
    (0x0040, 0x1103), # Person Telephone Numbers
    (0x4008, 0x0114), # Physician Approving Interpretation
    (0x0008, 0x1062), # Physician(s) Reading Study Identification Sequence
    (0x0008, 0x1048), # Physician(s) of Record
    (0x0008, 0x1049), # Physician(s) of Record Identification Sequence
    (0x0018, 0x1004), # Plate ID
    (0x0040, 0x0012), # Pre-Medication
    (0x0010, 0x21C0), # Pregnancy Status
    (0x0040, 0x2001), # Reason for the Imaging Service Request
    (0x0032, 0x1030), # Reason for Study
    (0x0400, 0x0402), # Referenced Digital Signature Sequence
    (0x0038, 0x0004), # Referenced Patient Alias Sequence
    (0x0008, 0x1120), # Referenced Patient Sequence
    (0x0400, 0x0403), # Referenced SOP Instance MAC Sequence
    (0x0008, 0x0092), # Referring Physician's Address
    (0x0008, 0x0096), # Referring Physician's Identification Sequence
    (0x0008, 0x0094), # Referring Physician's Telephone Numbers
    (0x0010, 0x2152), # Region of Residence
    (0x0040, 0x0275), # Request Attributes Sequence
    (0x0032, 0x1070), # Requested Contrast Agent
    (0x0040, 0x1400), # Requested Procedure Comments
    (0x0040, 0x1001), # Requested Procedure ID
    (0x0040, 0x1005), # Requested Procedure Location
    (0x0032, 0x1032), # Requesting Physician
    (0x0032, 0x1033), # Requesting Service
    (0x0010, 0x2299), # Responsible Organization
    (0x0010, 0x2297), # Responsible Person
    (0x4008, 0x4000), # Results Comments
    (0x4008, 0x0118), # Results Distribution List Sequence
    (0x4008, 0x0042), # Results ID Issuer
    (0x0040, 0x4034), # Scheduled Human Performers Sequence
    (0x0038, 0x001E), # Scheduled Patient Institution Residence
    (0x0040, 0x000B), # Scheduled Performing Physician Identification Sequence
    (0x0040, 0x0006), # Scheduled Performing Physician Name
    (0x0040, 0x0004), # Scheduled Procedure Step End Date
    (0x0040, 0x0005), # Scheduled Procedure Step End Time
    (0x0040, 0x0007), # Scheduled Procedure Step Description
    (0x0040, 0x0011), # Scheduled Procedure Step Location
    (0x0040, 0x0002), # Scheduled Procedure Step Start Date
    (0x0040, 0x0003), # Scheduled Procedure Step Start Time
    (0x0040, 0x0001), # Scheduled Station AE Title
    (0x0040, 0x4027), # Scheduled Station Geographic Location Code Sequence
    (0x0040, 0x0010), # Scheduled Station Name
    (0x0040, 0x4025), # Scheduled Station Name Code Sequence
    (0x0032, 0x1020), # Scheduled Study Location
    (0x0032, 0x1021), # Scheduled Study Location AE Title
    (0x0008, 0x103E), # Series Description
    (0x0038, 0x0062), # Service Episode Description
    (0x0038, 0x0060), # Service Episode ID
    (0x0010, 0x21A0), # Smoking Status
    (0x0038, 0x0050), # Special Needs
    (0x0032, 0x4000), # Study Comments
    (0x0008, 0x1030), # Study Description
    (0x0032, 0x0012), # Study ID Issuer
    (0x4000, 0x4000), # Text Comments
    (0x2030, 0x0020), # Text String
    (0x0008, 0x0201), # Timezone Offset From UTC
    (0x0088, 0x0910), # Topic Author
    (0x0088, 0x0912), # Topic Keywords
    (0x0088, 0x0906), # Topic Subject
    (0x0088, 0x0904), # Topic Title
    (0x0040, 0xA027), # Verifying Organization
    (0x0038, 0x4000), # Visit Comments
]

# Replace UID
U_TAGS = [
    (0x0020, 0x9161), # Concatenation UID
    (0x0008, 0x010D), # Context Group Extension Creator UID
    (0x0008, 0x9123), # Creator Version UID
    (0x0018, 0x1002), # Device UID
    (0x0020, 0x9164), # Dimension Organization UID
    (0x300A, 0x0013), # Dose Reference UID
    (0x0008, 0x0058), # Failed SOP Instance UID List
    (0x0070, 0x031A), # Fiducial UID
    (0x0020, 0x0052), # Frame of Reference UID
    (0x0008, 0x0014), # Instance Creator UID
    (0x0008, 0x3010), # Irradiation Event UID
    (0x0028, 0x1214), # Large Palette Color Lookup Table UID
    (0x0002, 0x0003), # Media Storage SOP Instance UID
    (0x0028, 0x1199), # Palette Color Lookup Table UID
    (0x3006, 0x0024), # Referenced Frame of Reference UID
    (0x0040, 0x4023), # Referenced General Purpose Scheduled Procedure Step Transaction UID
    (0x0008, 0x1155), # Referenced SOP Instance UID
    (0x0004, 0x1511), # Referenced SOP Instance UID in File
    (0x3006, 0x00C2), # Related Frame of Reference UID
    (0x0000, 0x1001), # Requested SOP Instance UID
    (0x0020, 0x000E), # Series Instance UID
    (0x0008, 0x0018), # SOP Instance UID
    (0x0088, 0x0140), # Storage Media File-set UID
    (0x0020, 0x000D), # Study Instance UID
    (0x0020, 0x0200), # Synchronization Frame of Reference UID
    (0x0040, 0xDB0D), # Template Extension Creator UID
    (0x0040, 0xDB0C), # Template Extension Organization UID
    (0x0008, 0x1195), # Transaction UID
    (0x0040, 0xA124), # UID
]

# Replace element according to the VR
Z_D_TAGS = [
    (0x0008, 0x0023), # Content Date
    (0x0008, 0x0033), # Content Time
    (0x0018, 0x0010), # Contrast Bolus Agent
]

# Set the value to empty according to the VR
X_Z_TAGS = [
    (0x0008, 0x0022), # Acquisition Date
    (0x0008, 0x0032), # Acquisition Time
    (0x0010, 0x2203), # Patient Sex Neutered
    (0x0008, 0x1110), # Referenced Study Sequence
    (0x0032, 0x1060), # Requested Procedure Description
    (0x300E, 0x0008), # Reviewer Name
]

# Replace element according to the VR
X_D_TAGS = [
    (0x0008, 0x002A), # Acquisition DateTime
    (0x0018, 0x1400), # Acquisition Device Processing Description
    (0x0018, 0x700A), # Detector ID
    (0x0008, 0x1072), # Operators' Identification Sequence
    (0x0018, 0x1030), # Protocol Name
    (0x0008, 0x0021), # Series Date
    (0x0008, 0x0031), # Series Time
]

# Replace element according to the VR
X_Z_D_TAGS = [
    (0x0018, 0x1000), # Device Serial Number
    (0x0008, 0x0082), # Institution Code Sequence
    (0x0008, 0x0080), # Institution Name
    (0x0008, 0x1070), # Operators' Name
    (0x0008, 0x1111), # Referenced Performed Procedure Step Sequence
    (0x0008, 0x1010), # Station Name
]

# Replace element with UI as VR, else replace according to VR with empty values
X_Z_U_STAR_TAGS = [
    (0x0008, 0x1140), # Referenced Image Sequence
    (0x0008, 0x2112), # Source Image Sequence
]



import re
import pydicom
import sys

from random import randint

from .dicomfields import *


dictionary = {}

# Default anonymization functions

def replaceElementUID(element):
    if element.value not in dictionary:
        new_chars = [str(randint(0, 9)) if char.isalnum() else char for char in element.value]
        dictionary[element.value] = ''.join(new_chars)
    element.value = dictionary.get(element.value)


def replaceElementDate(element):
    element.value = '00010101'


def replaceElementDateTime(element):
    element.value = '00010101010101.000000+0000'


def replaceElement(element):
    if element.VR == 'DA':
        replaceElementDate(element)
    elif element.VR == 'TM':
        element.value = '000000.00'
    elif element.VR in ('LO', 'SH', 'PN', 'CS'):
        element.value = 'Anonymized'
    elif element.VR == 'UI':
        replaceElementUID(element)
    elif element.VR == 'UL':
        pass
    elif element.VR == 'IS':
        element.value = '0'
    elif element.VR == 'SS':
        element.value = 0
    elif element.VR == 'SQ':
        for subDataset in element.value:
            for subElement in subDataset.elements():
                replaceElement(subElement)
    elif element.VR == 'DT':
        replaceElementDateTime(element)
    else:
        raise NotImplementedError('Not anonymized. VR {} not yet implemented.'.format(element.VR))


def replace(dataset, tag):
    """
    D - replace with a non-zero length value that may be a dummy value and consistent with the
    VR
    """
    element = dataset.get(tag)
    if element is not None:
        replaceElement(element)


def emptyElement(element):
    if (element.VR in ('SH', 'PN', 'UI', 'LO', 'CS')):
        element.value = ''
    elif element.VR == 'DA':
        replaceElementDate(element)
    elif element.VR == 'TM':
        element.value = '000000.00'
    elif element.VR == 'UL':
        element.value = 0
    elif element.VR == 'SQ':
        for subDataset in element.value:
            for subElement in subDataset.elements():
                emptyElement(subElement)
    else:
        raise NotImplementedError('Not anonymized. VR {} not yet implemented.'.format(element.VR))


def empty(dataset, tag):
    """
    Z - replace with a zero length value, or a non-zero length value that may be a dummy value and
    consistent with the VR
    """
    element = dataset.get(tag)
    if element is not None:
        emptyElement(element)


def deleteElement(dataset, element):
    if element.VR == 'DA':
        replaceElementDate(element)
    elif element.VR == 'SQ':
        for subDataset in element.value:
            for subElement in subDataset.elements():
                deleteElement(subDataset, subElement)
    else:
        del dataset[element.tag]


def delete(dataset, tag):
    """X - remove"""
    def rangeCallback(dataset, dataElement):
        if dataElement.tag.group & tag[2] == tag[0] and dataElement.tag.element & tag[3] == tag[1]:
            deleteElement(dataset, dataElement)

    if (len(tag) > 2):  # Tag ranges
        dataset.walk(rangeCallback)
    else:  # Individual Tags
        element = dataset.get(tag)
        if element is not None:
            deleteElement(dataset, element)  # element.tag is not the same type as tag.


def keep(dataset, tag):
    """K - keep (unchanged for non-sequence attributes, cleaned for sequences)"""
    pass


def clean(dataset, tag):
    """
    C - clean, that is replace with values of similar meaning known not to contain identifying
    information and consistent with the VR
    """
    if dataset.get(tag) is not None:
        raise NotImplementedError('Tag not anonymized. Not yet implemented.')


def replaceUID(dataset, tag):
    """
    U - replace with a non-zero length UID that is internally consistent within a set of Instances
    Lazy solution : Replace with empty string
    """
    element = dataset.get(tag)
    if element is not None:
        replaceElementUID(element)


def emptyOrReplace(dataset, tag):
    """Z/D - Z unless D is required to maintain IOD conformance (Type 2 versus Type 1)"""
    replace(dataset, tag)


def deleteOrEmpty(dataset, tag):
    """X/Z - X unless Z is required to maintain IOD conformance (Type 3 versus Type 2)"""
    empty(dataset, tag)


def deleteOrReplace(dataset, tag):
    """X/D - X unless D is required to maintain IOD conformance (Type 3 versus Type 1)"""
    replace(dataset, tag)


def deleteOrEmptyOrReplace(dataset, tag):
    """
    X/Z/D - X unless Z or D is required to maintain IOD conformance (Type 3 versus Type 2 versus
    Type 1)
    """
    replace(dataset, tag)


def deleteOrEmptyOrReplaceUID(dataset, tag):
    """
    X/Z/U* - X unless Z or replacement of contained instance UIDs (U) is required to maintain IOD
    conformance (Type 3 versus Type 2 versus Type 1 sequences containing UID references)
    """
    element = dataset.get(tag)
    if element is not None:
        if element.VR == 'UI':
            replaceElementUID(element)
        else:
            emptyElement(element)

# Generation functions

actionsMapNameFunctions = {
    "replace": replace,
    "empty": empty,
    "delete": delete,
    "replaceUID": replaceUID,
    "emptyOrReplace": emptyOrReplace,
    "deleteOrEmpty": deleteOrEmpty,
    "deleteOrReplace": deleteOrReplace,
    "deleteOrEmptyOrReplace": deleteOrEmptyOrReplace,
    "deleteOrEmptyOrReplaceUID": deleteOrEmptyOrReplaceUID,
    "keep": keep
}

def generateActions(tagList, action):
    """Generate a dictionnary using list values as tag and assign the same value to all
    :type tagList: list
    """
    finalAction = action
    if not callable(action):
        finalAction = actionsMapNameFunctions[action] if action in actionsMapNameFunctions else keep
    return {tag: finalAction for tag in tagList}


def initializeActions():
    """Initialize anonymization actions with DICOM standard values
    """
    anonymizationActions = generateActions(D_TAGS, replace)
    anonymizationActions.update(generateActions(Z_TAGS, empty))
    anonymizationActions.update(generateActions(X_TAGS, delete))
    anonymizationActions.update(generateActions(U_TAGS, replaceUID))
    anonymizationActions.update(generateActions(Z_D_TAGS, emptyOrReplace))
    anonymizationActions.update(generateActions(X_Z_TAGS, deleteOrEmpty))
    anonymizationActions.update(generateActions(X_D_TAGS, deleteOrReplace))
    anonymizationActions.update(generateActions(X_Z_D_TAGS, deleteOrEmptyOrReplace))
    anonymizationActions.update(generateActions(X_Z_U_STAR_TAGS, deleteOrEmptyOrReplaceUID))
    return anonymizationActions

def anonymizeDICOMFile(inFile, outFile, dictionary = ''):
    """Anonymize a DICOM file by modyfying personal tags

    Conforms to DICOM standard except for customer specificities.

    :param inFile: File path or file-like object to read from
    :param outFile: File path or file-like object to write to
    :param dictionary: add more tag's actions
    """
    currentAnonymizationActions = initializeActions()

    if dictionary != '':
        currentAnonymizationActions.update(dictionary)

    dataset = pydicom.dcmread(inFile)

    for tag, action in currentAnonymizationActions.items():
        action(dataset, tag)

    # X - Private tags = (0xgggg, oxeeee) where 0xgggg is odd
    dataset.remove_private_tags()

    # Store modified image
    dataset.save_as(outFile)



"""A setuptools based setup module.
See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages, setuptools
from os import path
# io.open is needed for projects that support Python 2.7
# It ensures open() defaults to text mode with universal newlines,
# and accepts an argument to specify the text encoding
# Python 3 only projects can skip this import
# from io import open

# here = path.abspath(path.dirname(__file__))

# # Get the long description from the README file
# with open(path.join(here, 'README.md'), encoding='utf-8') as f:
#     long_description = f.read()

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='dicom_anonymizer',  # Required
    version='1.0.4',  # Required
    description='Program to anonymize dicom files with default and custom rules', # Optional
    long_description=long_description,  # Optional
    long_description_content_type='text/markdown',  # Optional (see note above)ls
    url='https://gitlab.kitware.io/kitware/DicomAnonymizer.git',  # Optional
    author='Laurenn Lam',  # Optional
    author_email='laurenn.lam@kitware.com',  # Optional

    # Classifiers help users find your project by categorizing it.
    #
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[  # Optional
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python'
    ],

    # This field adds keywords for your project which will appear on the
    # project page. What does your project relate to?
    #
    # Note that this is a string of words separated by whitespace, not a list.
    keywords='dicom anonymizer medical',  # Optional

    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    packages=setuptools.find_packages(),  # Required

    # Define an executable calls dicom-anonymizer from a specific file
    entry_points={
        'console_scripts': [
            'dicom-anonymizer = dicomanonymizer.anonymizer:main'
        ]
    },

    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['pydicom', 'tqdm'],  # Optional
)